"""
Screen and camera capture functionality for FMUS-VID.

This module provides tools for capturing screenshots, recording screen activity,
and capturing video from webcams and camera devices.
"""

from .screen import capture_screen, capture_region, select_region
from .recorder import record_screen, record_region
from .camera import CameraCapture, list_cameras, get_camera_backends

__all__ = [
    # Screen capture
    'capture_screen',
    'capture_region',
    'record_screen',
    'record_region',
    'select_region',

    # Camera capture
    'CameraCapture',
    'list_cameras',
    'get_camera_backends',
]
