"""
Home Assistant Integration Package for Speech Recognition.

This package provides integrations with external speech recognition systems,
including Home Assistant's Faster Whisper integration.
"""

# Import availability flag
try:
    from .home_assistant import HOME_ASSISTANT_AVAILABLE
except ImportError:
    HOME_ASSISTANT_AVAILABLE = False

__all__ = ["HOME_ASSISTANT_AVAILABLE"]
